from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowAction
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
import string

factory_type_information = (
    {'id': 'MailMessage',
     'content_icon': 'mail.png',
     'meta_type': 'CMF MailMessage',
     'description': ('MailMessages are object that store mail messages'
                     'that have arrived in the Portal'),
     'product': 'CMFMailIn',
     'factory': 'addMailMessage',
     'immediate_view': 'mailmessage_view',
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'mailmessage_view',
                  'permissions': (CMFCorePermissions.View,)},)
     },
    )
                            
def addMailMessage(self,
                   id,
                   title = '',
                   subject = '',
                   date = '',
                   to = '',
                   sender = '',
                   replyto = '',
                   body = '',
                   headers = '',
                   otherInfo = '',
                   attachments = {},
                   REQUEST = None,
                   **kw):
    
    """
    Create an empty MailMessage.
    """
    mm = MailMessage(id,
                     title,
                     subject,
                     date,
                     to,
                     sender,
                     replyto,
                     body,
                     headers,
                     otherInfo,
                     attachments,
                     **kw
                     )
    self._setObject(id, mm)
    

class MailMessage(PortalContent, DefaultDublinCoreImpl):
    """
    MailMessage is a Mail Message. Pretty obvious really
    """
    meta_type='CMF MailMessage'

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)

    def __init__(self,
                 id,
                 title = '',
                 subject = '',
                 date = '',
                 to = '',
                 sender = '',
                 replyto = '',
                 body = '',
                 headers = '',
                 otherInfo = '',
                 attachments = {},
                 **kw
                 ):

        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.setTitle(title)
        self.setSubject(subject)
        self.date = date
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers
        self.otherInfo = otherInfo
        self.attachments = attachments

    security.declareProtected(CMFCorePermissions.View, 'Subject')        
    def Subject(self):
        return string.join(self.subject,' ')

    security.declareProtected(CMFCorePermissions.View, 'Date')        
    def Date(self):
        return self.date
    
    security.declareProtected(CMFCorePermissions.View, 'To')        
    def To(self):
        return self.to

    security.declareProtected(CMFCorePermissions.View, 'Sender')
    def Sender(self):
        return self.sender

    security.declareProtected(CMFCorePermissions.View, 'ReplyTo')
    def ReplyTo(self):
        return self.replyto

    security.declareProtected(CMFCorePermissions.View, 'Body')
    def Body(self):
        return self.body

    security.declareProtected(CMFCorePermissions.View, 'Headers')
    def Headers(self):
        return self.headers

    security.declareProtected(CMFCorePermissions.View, 'OtherInfo')
    def OtherInfo(self):
        return self.otherInfo

    security.declareProtected(CMFCorePermissions.View, 'Attachments')
    def Attachments(self):
        return self.attachments.keys()
    
    security.declareProtected(CMFCorePermissions.View, 'getAttachment')
    def getAttachment(self, key):
        """ return the attachment """
        import mimetypes
        mimetype = ()
        mimetype = mimetypes.guess_type(key)[0]
        if not mimetype:
            mimetype = 'application/octet-stream'
        self.REQUEST.RESPONSE.setHeader('Content-Type', mimetype)
        return self.attachments[key]
        

    def _convertToSortableDate(self, date):
        import rfc822
        # Converts a date in to a string which can be sorted to produce the correct results
        if not date:
            return ""
        dateTuple = rfc822.parsedate(date)
        return str('%04d%02d%02d%02d%02d%02d' % (dateTuple[0],dateTuple[1],dateTuple[2],dateTuple[3],dateTuple[4],dateTuple[5]))
    